@@ -1,10 +1,12 @@
 from __future__ import absolute_import, division, print_function
 
 from itertools import product
+from operator import getitem
 
 import numpy as np
 
-from .core import normalize_chunks, Array
+from .core import (normalize_chunks, Array, slices_from_chunks,
+                   broadcast_shapes, broadcast_to)
 from ..base import tokenize
 from ..utils import different_seeds, ignoring
 
@@ -15,7 +17,6 @@ def doc_wraps(func):
         return func2
     return _
 
-
 class RandomState(object):
     """
     Mersenne Twister pseudo-random number generator
@@ -48,31 +49,101 @@ class RandomState(object):
 
         extra_chunks should be a chunks tuple to append to the end of chunks
         """
-        size = kwargs.pop('size')
+        size = kwargs.pop('size', None)
         chunks = kwargs.pop('chunks')
         extra_chunks = kwargs.pop('extra_chunks', ())
 
-        if not isinstance(size, (tuple, list)):
+        if size is not None and not isinstance(size, (tuple, list)):
             size = (size,)
 
+        args_shapes = {ar.shape for ar in args
+                       if isinstance(ar, (Array, np.ndarray))}
+        args_shapes.union({ar.shape for ar in kwargs.values()
+                           if isinstance(ar, (Array, np.ndarray))})
+
+        shapes = list(args_shapes)
+        if size is not None:
+            shapes += [size]
+        # broadcast to the final size(shape)
+        size = broadcast_shapes(*shapes)
         chunks = normalize_chunks(chunks, size)
+        slices = slices_from_chunks(chunks)
+
+        def _broadcast_any(ar, shape, chunks):
+            if isinstance(ar, Array):
+                return broadcast_to(ar, shape).rechunk(chunks)
+            if isinstance(ar, np.ndarray):
+                return np.ascontiguousarray(np.broadcast_to(ar, shape))
+
+        # Broadcast all arguments, get tiny versions as well
+        # Start adding the relevant bits to the graph
+        dsk = {}
+        lookup = {}
+        small_args = []
+        for i, ar in enumerate(args):
+            if isinstance(ar, (np.ndarray, Array)):
+                res = _broadcast_any(ar, size, chunks)
+                if isinstance(res, Array):
+                    dsk.update(res.dask)
+                    lookup[i] = res.name
+                elif isinstance(res, np.ndarray):
+                    name = 'array-{}'.format(tokenize(res))
+                    lookup[i] = name
+                    dsk[name] = res
+                small_args.append(ar[tuple(0 for _ in ar.shape)])
+            else:
+                small_args.append(ar)
+
+        small_kwargs = {}
+        for key, ar in kwargs.items():
+            if isinstance(ar, (np.ndarray, Array)):
+                res = _broadcast_any(ar, size, chunks)
+                if isinstance(res, Array):
+                    dsk.update(res.dask)
+                    lookup[key] = res.name
+                elif isinstance(res, np.ndarray):
+                    name = 'array-{}'.format(tokenize(res))
+                    lookup[key] = name
+                    dsk[name] = res
+                small_kwargs[key] = ar[tuple(0 for _ in ar.shape)]
+            else:
+                small_kwargs[key] = ar
 
         # Get dtype
-        kw = kwargs.copy()
-        kw['size'] = (0,)
-        dtype = func(np.random.RandomState(), *args, **kw).dtype
+        small_kwargs['size'] = (0,)
+        dtype = func(np.random.RandomState(), *small_args,
+                     **small_kwargs).dtype
 
-        # Build graph
         sizes = list(product(*chunks))
         seeds = different_seeds(len(sizes), self._numpy_state)
         token = tokenize(seeds, size, chunks, args, kwargs)
         name = 'da.random.{0}-{1}'.format(func.__name__, token)
+
         keys = product([name], *([range(len(bd)) for bd in chunks]
                                + [[0]] * len(extra_chunks)))
-        vals = ((_apply_random, func.__name__, seed, size, args, kwargs)
-                for seed, size in zip(seeds, sizes))
-        dsk = dict(zip(keys, vals))
-
+        blocks = product(*[range(len(bd)) for bd in chunks])
+        vals = []
+        for seed, size, slc, block in zip(seeds, sizes, slices, blocks):
+            arg = []
+            for i, ar in enumerate(args):
+                if i not in lookup:
+                    arg.append(ar)
+                else:
+                    if isinstance(ar, Array):
+                        arg.append((lookup[i], ) + block)
+                    else:   # np.ndarray
+                        arg.append((getitem, lookup[i], slc))
+            kwrg = {}
+            for k, ar in kwargs.items():
+                if k not in lookup:
+                    kwrg[k] = ar
+                else:
+                    if isinstance(ar, Array):
+                        kwrg[k] = (lookup[k], ) + block
+                    else:   # np.ndarray
+                        kwrg[k] = (getitem, lookup[k], slc)
+            vals.append((_apply_random, func.__name__, seed, size, arg, kwrg))
+        dsk.update(dict(zip(keys, vals)))
         return Array(dsk, name, chunks + extra_chunks, dtype=dtype)
 
     @doc_wraps(np.random.RandomState.beta)
