@@ -5,10 +5,11 @@ import warnings
 import numpy as np
 import pandas as pd
 
-from dask.dataframe.core import (DataFrame, Series,
+from dask.dataframe.core import (DataFrame, Series, Index,
                                  aca, map_partitions, no_default)
 from dask.utils import derived_from
 
+
 def _maybe_slice(grouped, columns):
     """
     Slice columns if grouped is pd.DataFrameGroupBy
@@ -18,16 +19,19 @@ def _maybe_slice(grouped, columns):
             return grouped[columns]
     return grouped
 
+
 def _groupby_apply_level0(df, key, func):
     grouped = df.groupby(level=0)
     grouped = _maybe_slice(grouped, key)
     return grouped.apply(func)
 
+
 def _groupby_apply_index(df, ind, key, func):
     grouped = df.groupby(ind)
     grouped = _maybe_slice(grouped, key)
     return grouped.apply(func)
 
+
 def _groupby_get_group(df, by_key, get_key, columns):
     # SeriesGroupBy may pass df which includes group key
     grouped = df.groupby(by_key)
@@ -56,18 +60,53 @@ def _apply_chunk(df, index, func, columns):
     else:
         return func(df.groupby(index)[columns])
 
+
 def _sum(g):
     return g.sum()
 
+
 def _min(g):
     return g.min()
 
+
 def _max(g):
     return g.max()
 
+
 def _count(g):
     return g.count()
 
+
+def _var_chunk(df, index):
+    if isinstance(df, pd.Series):
+        df = df.to_frame()
+    x = df.groupby(index).sum()
+    x2 = (df**2).rename(columns=lambda c: c + '-x2')
+    cols = [c + '-x2' for c in x.columns]
+    x2 = pd.concat([df, x2], axis=1).groupby(index)[cols].sum()
+    n = (df.groupby(index).count()
+           .rename(columns=lambda c: c + '-count'))
+
+    result = pd.concat([x, x2, n], axis=1)
+    return result
+
+
+def _var_agg(g, ddof):
+    g = g.groupby(level=0).sum()
+    nc = len(g.columns)
+    x = g[g.columns[:nc//3]]
+    x2 = g[g.columns[nc//3:2*nc//3]].rename(columns=lambda c: c[:-3])
+    n = g[g.columns[-nc//3:]].rename(columns=lambda c: c[:-6])
+
+    result = x2 - x**2 / n
+    div = (n - ddof)
+    div[div < 0] = 0
+    result /= div
+    result[(n - ddof) == 0] = np.nan
+    assert isinstance(result, pd.DataFrame)
+    return result
+
+
 ###############################################################
 # nunique
 ###############################################################
@@ -79,6 +118,7 @@ def _nunique_df_chunk(df, index):
     grouped.index = grouped.index.get_level_values(level=0)
     return grouped
 
+
 def _nunique_series_chunk(df, index):
     assert isinstance(df, pd.Series)
     if isinstance(index, np.ndarray):
@@ -107,6 +147,17 @@ class _GroupBy(object):
         self.obj = df
 
         # grouping key passed via groupby method
+        if (isinstance(index, (DataFrame, Series, Index)) and
+            isinstance(df, DataFrame)):
+
+            if (isinstance(index, Series) and index.name in df.columns and
+                index._name == df[index.name]._name):
+                index = index.name
+            elif (isinstance(index, DataFrame) and
+                set(index.columns).issubset(df.columns) and
+                index._name == df[index.columns]._name):
+                index = list(index.columns)
+
         self.index = index
 
         # slicing key applied to _GroupBy instance
@@ -138,10 +189,16 @@ class _GroupBy(object):
         """
         Return whether index is a Series sliced from df
         """
-        if isinstance(df, DataFrame) and isinstance(index, Series):
-            if (index.name is not None and
-                index._name == df._name + '.' + index.name):
-                return True
+        if isinstance(df, Series):
+            return False
+        if (isinstance(index, Series) and index._name in df.columns and
+            index._name == df[index.name]._name):
+            return True
+        if (isinstance(index, DataFrame) and
+            set(index.columns).issubset(df.columns) and
+            index._name == df[index.columns]._name):
+            index = list(index.columns)
+            return True
         return False
 
     def _head(self):
@@ -168,7 +225,7 @@ class _GroupBy(object):
 
         token = self._token_prefix + token
 
-        if isinstance(self.index, list):
+        if isinstance(self.index, (tuple, list)) and len(self.index) > 1:
             levels = list(range(len(self.index)))
         else:
             levels = 0
@@ -200,6 +257,30 @@ class _GroupBy(object):
     def mean(self):
         return self.sum() / self.count()
 
+    @derived_from(pd.core.groupby.GroupBy)
+    def var(self, ddof=1):
+        from functools import partial
+        meta = self.obj._pd
+        if isinstance(meta, pd.Series):
+            meta = meta.to_frame()
+        meta = meta.groupby(self.index).var(ddof=1)
+        result = aca([self.obj, self.index], _var_chunk,
+                     partial(_var_agg, ddof=ddof), meta,
+                     token=self._token_prefix + 'var')
+
+        if isinstance(self.obj, Series):
+            result = result[result.columns[0]]
+        if self._slice:
+            result = result[self._slice]
+
+        return result
+
+    @derived_from(pd.core.groupby.GroupBy)
+    def std(self, ddof=1):
+        v = self.var(ddof)
+        result = map_partitions(np.sqrt, v, v)
+        return result
+
     @derived_from(pd.core.groupby.GroupBy)
     def get_group(self, key):
         token = self._token_prefix + 'get_group'
@@ -230,7 +311,7 @@ class _GroupBy(object):
             If list is given, the result is a DataFrame which columns is
             specified list. Otherwise, the result is a Series which name is
             given scalar or None (no name). If name keyword is not given, dask
-            tries to infer the result type using its beggining of data. This
+            tries to infer the result type using its beginning of data. This
             inference may take some time and lead to unexpected result
 
         Returns
@@ -302,7 +383,7 @@ class DataFrameGroupBy(_GroupBy):
 
     def __dir__(self):
         return sorted(set(dir(type(self)) + list(self.__dict__) +
-                      list(self.obj.columns)))
+                      list(filter(pd.compat.isidentifier, self.obj.columns))))
 
     def __getattr__(self, key):
         try:
