@@ -7,21 +7,26 @@ from math import ceil
 from operator import getitem
 import os
 from threading import Lock
+import multiprocessing
 import uuid
 from warnings import warn
 
 import pandas as pd
 import numpy as np
+import dask
 from toolz import merge
 
 from ..base import tokenize
 from ..compatibility import unicode, apply
 from .. import array as da
 from ..async import get_sync
+from ..delayed import Delayed, delayed
+import dask.multiprocessing
 
 from .core import _Frame, DataFrame, Series
 from .shuffle import set_partition
 
+from ..utils import build_name_function
 
 lock = Lock()
 
@@ -158,16 +163,20 @@ def from_pandas(data, npartitions=None, chunksize=None, sort=True, name=None):
     else:
         npartitions = int(ceil(nrows / chunksize))
 
+    name = name or ('from_pandas-' + tokenize(data, chunksize))
+
+    if not nrows:
+        return _Frame({(name, 0): data}, name, data, [None, None])
+
     if sort and not data.index.is_monotonic_increasing:
         data = data.sort_index(ascending=True)
     if sort:
         divisions, locations = sorted_division_locations(data.index,
                                                          chunksize=chunksize)
     else:
-        divisions = [None] * (npartitions + 1)
         locations = list(range(0, nrows, chunksize)) + [len(data)]
+        divisions = [None] * len(locations)
 
-    name = name or ('from_pandas-' + tokenize(data, chunksize))
     dsk = dict(((name, i), data.iloc[start: stop])
                for i, (start, stop) in enumerate(zip(locations[:-1],
                    locations[1:])))
@@ -237,7 +246,8 @@ def from_bcolz(x, chunksize=None, categorize=True, index=None, lock=lock,
                  columns, categories, lock))
                for i in range(0, int(ceil(len(x) / chunksize))))
 
-    result = DataFrame(dsk, new_name, columns, divisions)
+    meta = dataframe_from_ctable(x, slice(0, 0), columns, categories, lock)
+    result = DataFrame(dsk, new_name, meta, divisions)
 
     if index:
         assert index in x.names
@@ -397,35 +407,110 @@ def _link(token, result):
     return None
 
 
+def _pd_to_hdf(pd_to_hdf, lock, args, kwargs=None):
+    """ A wrapper function around pd_to_hdf that enables locking"""
+
+    if lock:
+        lock.acquire()
+    try:
+        pd_to_hdf(*args, **kwargs)
+    finally:
+        if lock:
+            lock.release()
+
+    return None
+
+
 @wraps(pd.DataFrame.to_hdf)
 def to_hdf(df, path_or_buf, key, mode='a', append=False, complevel=0,
-           complib=None, fletcher32=False, get=get_sync, dask_kwargs=None,
-           **kwargs):
+           complib=None, fletcher32=False, get=None, dask_kwargs={},
+           name_function=None, compute=True, lock=None, **kwargs):
     name = 'to-hdf-' + uuid.uuid1().hex
 
     pd_to_hdf = getattr(df._partition_type, 'to_hdf')
 
+    single_file = True
+    single_node = True
+
+    # if path_or_buf is string, format using i_name
+    if isinstance(path_or_buf, str):
+        if path_or_buf.count('*') + key.count('*') > 1:
+            raise ValueError("A maximum of one asterisk is accepted in file path and dataset key")
+
+        fmt_obj = lambda path_or_buf, i_name: path_or_buf.replace('*', i_name)
+
+        if '*' in path_or_buf:
+            single_file = False
+    else:
+        if key.count('*') > 1:
+            raise ValueError("A maximum of one asterisk is accepted in dataset key")
+
+        fmt_obj = lambda path_or_buf, _: path_or_buf
+
+    if '*' in key:
+        single_node = False
+
+    if name_function is None:
+        name_function = build_name_function(df.npartitions - 1)
+
+    # we guarantee partition order is preserved when its saved and read
+    # so we enforce name_function to maintain the order of its input.
+    if not (single_file and single_node):
+        formatted_names = [name_function(i) for i in range(df.npartitions)]
+        if formatted_names != sorted(formatted_names):
+            warn("To preserve order between partitions name_function "
+                 "must preserve the order of its input")
+
+    # handle lock default based on wether we're writing to a single entity
+    if lock is None:
+        if not single_node or not single_file:
+            lock = True
+        else:
+            lock = False
+
+    if lock is True:
+        _actual_get = get if get else dask.get
+        if _actual_get == dask.multiprocessing.get:
+            lock = multiprocessing.Manager().Lock()
+        else:
+            lock = Lock()
+
+    kwargs.update({'format': 'table', 'complevel': complevel,
+                   'complib': complib, 'fletcher32': fletcher32,
+                   'mode': mode, 'append': append})
     dsk = dict()
-    dsk[(name, 0)] = (_link, None,
-                      (apply, pd_to_hdf,
-                          (tuple, [(df._name, 0), path_or_buf, key]),
-                          merge(kwargs,
-                            {'mode':  mode, 'format': 'table', 'append': append,
-                             'complevel': complevel, 'complib': complib,
-                             'fletcher32': fletcher32})))
+
+    i_name = name_function(0)
+    dsk[(name, 0)] = (_pd_to_hdf, pd_to_hdf, lock,
+                      [(df._name, 0), fmt_obj(path_or_buf, i_name),
+                             key.replace('*', i_name)], kwargs)
+
+    kwargs2 = kwargs.copy()
+    if single_file:
+        kwargs2['mode'] = 'a'
+    if single_node:
+        kwargs2['append'] = True
+
     for i in range(1, df.npartitions):
-        dsk[(name, i)] = (_link, (name, i - 1),
-                          (apply, pd_to_hdf,
-                           (tuple, [(df._name, i), path_or_buf, key]),
-                           merge(kwargs,
-                             {'mode': 'a', 'format': 'table', 'append': True,
-                              'complevel': complevel, 'complib': complib,
-                              'fletcher32': fletcher32})))
+        i_name = name_function(i)
+        task = (_pd_to_hdf, pd_to_hdf, lock,
+                [(df._name, i), fmt_obj(path_or_buf, i_name),
+                    key.replace('*', i_name)], kwargs2)
+        if single_file:
+            link_dep = i - 1 if single_node else 0
+            task = (_link, (name, link_dep), task)
+        dsk[(name, i)] = task
 
-    dask_kwargs = dask_kwargs or {}
+    dsk = merge(df.dask, dsk)
+    if single_file and single_node:
+        keys = [(name, df.npartitions - 1)]
+    else:
+        keys = [(name, i) for i in range(df.npartitions)]
 
-    DataFrame._get(merge(df.dask, dsk), (name, df.npartitions - 1),
-                   get=get, **dask_kwargs)
+    if compute:
+        return DataFrame._get(dsk, keys, get=get, **dask_kwargs)
+    else:
+        return delayed([Delayed(key, [dsk]) for key in keys])
 
 
 dont_use_fixed_error_message = """
@@ -490,9 +575,6 @@ def _read_single_hdf(path, key, start=0, stop=None, columns=None,
         divisions = [None] * (len(dsk) + 1)
         return DataFrame(dsk, name, empty, divisions)
 
-    if lock is True:
-        lock = Lock()
-
     keys, stops = get_keys_and_stops(path, key, stop)
     if (start != 0 or stop is not None) and len(keys) > 1:
         raise NotImplementedError(read_hdf_error_msg)
@@ -550,6 +632,9 @@ def read_hdf(pattern, key, start=0, stop=None, columns=None,
 
     >>> dd.read_hdf('myfile.1.hdf5', '/*')  # doctest: +SKIP
     """
+    if lock is True:
+        lock = Lock()
+
     key = key if key.startswith('/')  else '/' + key
     paths = sorted(glob(pattern))
     if (start != 0 or stop is not None) and len(paths) > 1:
@@ -592,31 +677,62 @@ def to_castra(df, fn=None, categories=None, sorted_index_column=None,
     dsk = merge(dsk, df.dask)
     keys = [(name, -1), (name, df.npartitions - 1)]
     if compute:
-        c, _ = DataFrame._get(dsk, keys, get=get)
-        return c
+        return DataFrame._get(dsk, keys, get=get)[0]
     else:
-        return dsk, keys
+        return delayed([Delayed(key, [dsk]) for key in keys])[0]
 
 
-def to_csv(df, filename, compression=None, get=None, **kwargs):
+def to_csv(df, filename, name_function=None, compression=None, get=None, compute=True, **kwargs):
     if compression:
         raise NotImplementedError("Writing compressed csv files not supported")
     name = 'to-csv-' + uuid.uuid1().hex
 
+    kwargs2 = kwargs.copy()
+
+    if name_function is None:
+        name_function = build_name_function(df.npartitions - 1)
+
+    if '*' in filename:
+        if filename.count('*') > 1:
+            raise ValueError("A maximum of one asterisk is accepted in filename")
+
+        if 'mode' in kwargs and kwargs['mode'] != 'w':
+            raise ValueError("to_csv does not support writing to multiple files in append mode, "
+                             "please specify mode='w'")
+
+        formatted_names = [name_function(i) for i in range(df.npartitions)]
+        if formatted_names != sorted(formatted_names):
+            warn("To preserve order between partitions name_function "
+                 "must preserve the order of its input")
+
+        single_file = False
+    else:
+        kwargs2.update({'mode': 'a', 'header': False})
+        single_file = True
+
     dsk = dict()
     dsk[(name, 0)] = (lambda df, fn, kwargs: df.to_csv(fn, **kwargs),
-                        (df._name, 0), filename, kwargs)
-
-    kwargs2 = kwargs.copy()
-    kwargs2['mode'] = 'a'
-    kwargs2['header'] = False
+                        (df._name, 0), filename.replace('*', name_function(0)), kwargs)
 
     for i in range(1, df.npartitions):
-        dsk[(name, i)] = (_link, (name, i - 1),
-                           (lambda df, fn, kwargs: df.to_csv(fn, **kwargs),
-                             (df._name, i), filename, kwargs2))
+        filename_i = filename.replace('*', name_function(i))
 
-    DataFrame._get(merge(dsk, df.dask), (name, df.npartitions - 1), get=get)
+        task = (lambda df, fn, kwargs: df.to_csv(fn, **kwargs),
+                 (df._name, i), filename_i, kwargs2)
+        if single_file:
+            task = (_link, (name, i - 1), task)
+        dsk[(name, i)] = task
+
+    dsk = merge(dsk, df.dask)
+    if single_file:
+        keys = [(name, df.npartitions - 1)]
+    else:
+        keys = [(name, i) for i in range(df.npartitions)]
+
+    if compute:
+        return DataFrame._get(dsk, keys, get=get)
+    else:
+        return delayed([Delayed(key, [dsk]) for key in keys])
 
 
 def to_bag(df, index=False):
@@ -645,16 +761,24 @@ def from_delayed(dfs, metadata=None, divisions=None, columns=None,
     Parameters
     ----------
     dfs: list of Values
-        An iterable of dask.delayed.Value objects, such as come from dask.do
-        These comprise the individual partitions of the resulting dataframe
-    metadata: list or string of column names or empty dataframe
-    divisions: list or None
+        An iterable of ``dask.delayed.Delayed`` objects, such as come from
+        ``dask.delayed`` These comprise the individual partitions of the
+        resulting dataframe.
+    metadata: str, list of column names, or empty dataframe, optional
+        Metadata for the underlying pandas object. Can be either column name
+        (if Series), list of column names, or pandas object with the same
+        columns/dtypes. If not provided, will be computed from the first
+        partition.
+    divisions: list, optional
+        Partition boundaries along the index.
+    prefix, str, optional
+        Prefix to prepend to the keys.
     """
     if columns is not None:
         warn("Deprecation warning: Use metadata argument, not columns")
         metadata = columns
-    from dask.delayed import Value
-    if isinstance(dfs, Value):
+    from dask.delayed import Delayed
+    if isinstance(dfs, Delayed):
         dfs = [dfs]
     dsk = merge(df.dask for df in dfs)
 
@@ -665,8 +789,10 @@ def from_delayed(dfs, metadata=None, divisions=None, columns=None,
 
     if divisions is None:
         divisions = [None] * (len(dfs) + 1)
+    if metadata is None:
+        metadata = dfs[0].compute()
 
-    if isinstance(metadata, str):
+    if isinstance(metadata, (str, pd.Series)):
         return Series(merge(dsk, dsk2), name, metadata, divisions)
     else:
         return DataFrame(merge(dsk, dsk2), name, metadata, divisions)
