@@ -1,5 +1,6 @@
 from datetime import datetime
 
+import pytest
 import numpy as np
 import pandas as pd
 
@@ -8,6 +9,7 @@ import dask.dataframe as dd
 from dask.dataframe.utils import eq, assert_dask_graph
 
 
+@pytest.mark.slow
 def test_arithmetics():
     dsk = {('x', 0): pd.DataFrame({'a': [1, 2, 3], 'b': [4, 5, 6]},
                                   index=[0, 1, 3]),
@@ -109,6 +111,7 @@ def test_arithmetics():
                                 allow_comparison_ops=False)
 
 
+@pytest.mark.slow
 def test_arithmetics_different_index():
     # index are different, but overwraps
     pdf1 = pd.DataFrame({'a': [1, 2, 3, 4, 5], 'b': [3, 5, 2, 5, 7]},
@@ -612,18 +615,22 @@ def test_reductions():
         assert eq(dds.var(skipna=False, ddof=0), pds.var(skipna=False, ddof=0))
         assert eq(dds.mean(skipna=False), pds.mean(skipna=False))
 
+
     assert_dask_graph(ddf1.b.sum(), 'series-sum')
     assert_dask_graph(ddf1.b.min(), 'series-min')
     assert_dask_graph(ddf1.b.max(), 'series-max')
     assert_dask_graph(ddf1.b.count(), 'series-count')
-    assert_dask_graph(ddf1.b.std(), 'series-std(ddof=1)')
-    assert_dask_graph(ddf1.b.var(), 'series-var(ddof=1)')
-    assert_dask_graph(ddf1.b.std(ddof=0), 'series-std(ddof=0)')
-    assert_dask_graph(ddf1.b.var(ddof=0), 'series-var(ddof=0)')
+    assert_dask_graph(ddf1.b.std(), 'series-std')
+    assert_dask_graph(ddf1.b.var(), 'series-var')
+    assert_dask_graph(ddf1.b.std(ddof=0), 'series-std')
+    assert_dask_graph(ddf1.b.var(ddof=0), 'series-var')
     assert_dask_graph(ddf1.b.mean(), 'series-mean')
     # nunique is performed using drop-duplicates
     assert_dask_graph(ddf1.b.nunique(), 'drop-duplicates')
 
+    eq(ddf1.index.min(), pdf1.index.min())
+    eq(ddf1.index.max(), pdf1.index.max())
+
 
 def test_reduction_series_invalid_axis():
     dsk = {('x', 0): pd.DataFrame({'a': [1, 2, 3], 'b': [4, 5, 6]},
@@ -747,13 +754,13 @@ def test_reductions_frame():
     assert_dask_graph(ddf1.mean(), 'dataframe-count')
 
     # axis=1
-    assert_dask_graph(ddf1.sum(axis=1), 'dataframe-sum(axis=1)')
-    assert_dask_graph(ddf1.min(axis=1), 'dataframe-min(axis=1)')
-    assert_dask_graph(ddf1.max(axis=1), 'dataframe-max(axis=1)')
-    assert_dask_graph(ddf1.count(axis=1), 'dataframe-count(axis=1)')
-    assert_dask_graph(ddf1.std(axis=1), 'dataframe-std(axis=1, ddof=1)')
-    assert_dask_graph(ddf1.var(axis=1), 'dataframe-var(axis=1, ddof=1)')
-    assert_dask_graph(ddf1.mean(axis=1), 'dataframe-mean(axis=1)')
+    assert_dask_graph(ddf1.sum(axis=1), 'dataframe-sum')
+    assert_dask_graph(ddf1.min(axis=1), 'dataframe-min')
+    assert_dask_graph(ddf1.max(axis=1), 'dataframe-max')
+    assert_dask_graph(ddf1.count(axis=1), 'dataframe-count')
+    assert_dask_graph(ddf1.std(axis=1), 'dataframe-std')
+    assert_dask_graph(ddf1.var(axis=1), 'dataframe-var')
+    assert_dask_graph(ddf1.mean(axis=1), 'dataframe-mean')
 
 
 def test_reductions_frame_dtypes():
