@@ -1,5 +1,4 @@
 from operator import getitem
-from distutils.version import LooseVersion
 
 import pandas as pd
 import pandas.util.testing as tm
@@ -156,6 +155,52 @@ def test_set_index():
     assert eq(d4, full.set_index('b'))
 
 
+def test_set_index_interpolate():
+    df = pd.DataFrame({'x': [1, 1, 1, 3, 3], 'y': [1., 1, 1, 1, 2]})
+    d = dd.from_pandas(df, 2)
+
+    d1 = d.set_index('x', npartitions=3)
+    assert d1.npartitions == 3
+    assert set(d1.divisions) == set([1, 2, 3])
+
+    d2 = d.set_index('y', npartitions=3)
+    assert d2.divisions[0] == 1.
+    assert 1. < d2.divisions[1] < d2.divisions[2] < 2.
+    assert d2.divisions[3] == 2.
+
+
+def test_set_index_interpolate_int():
+    L = sorted(list(range(0, 200, 10))*2)
+    df = pd.DataFrame({'x': 2*L})
+    d = dd.from_pandas(df, 2)
+    d1 = d.set_index('x', npartitions=10)
+    assert all(np.issubdtype(type(x), np.integer) for x in d1.divisions)
+
+
+def test_set_index_timezone():
+    s_naive = pd.Series(pd.date_range('20130101', periods=3))
+    s_aware = pd.Series(pd.date_range('20130101', periods=3, tz='US/Eastern'))
+    df = pd.DataFrame({'tz': s_aware, 'notz': s_naive})
+    d = dd.from_pandas(df, 2)
+
+    d1 = d.set_index('notz', npartitions=2)
+    s1 = pd.DatetimeIndex(s_naive.values, dtype=s_naive.dtype)
+    assert d1.divisions[0] == s_naive[0] == s1[0]
+    assert d1.divisions[2] == s_naive[2] == s1[2]
+
+    # We currently lose "freq".  Converting data with pandas-defined dtypes
+    # to numpy or pure Python can be lossy like this.
+    d2 = d.set_index('tz', npartitions=2)
+    s2 = pd.DatetimeIndex(s_aware.values, dtype=s_aware.dtype)
+    assert d2.divisions[0] == s2[0]
+    assert d2.divisions[2] == s2[2]
+    assert d2.divisions[0].tz == s2[0].tz
+    assert d2.divisions[0].tz is not None
+    s2badtype = pd.DatetimeIndex(s_aware.values, dtype=s_naive.dtype)
+    with pytest.raises(TypeError):
+        d2.divisions[0] == s2badtype[0]
+
+
 @pytest.mark.parametrize('drop', [True, False])
 def test_set_index_drop(drop):
 
@@ -444,12 +489,7 @@ def test_drop_duplicates_subset():
                        'y': ['a', 'a', 'b', 'b', 'c', 'c']})
     ddf = dd.from_pandas(df, npartitions=2)
 
-    if pd.__version__ < '0.17':
-        kwargs = [{'take_last': False}, {'take_last': True}]
-    else:
-        kwargs = [{'keep': 'first'}, {'keep': 'last'}]
-
-    for kwarg in kwargs:
+    for kwarg in [{'keep': 'first'}, {'keep': 'last'}]:
         assert eq(df.x.drop_duplicates(**kwarg),
                   ddf.x.drop_duplicates(**kwarg))
         for ss in [['x'], 'y', ['x', 'y']]:
@@ -654,94 +694,6 @@ def test_index():
     assert eq(d.index, full.index)
 
 
-def test_loc():
-    assert d.loc[3:8].divisions[0] == 3
-    assert d.loc[3:8].divisions[-1] == 8
-
-    assert d.loc[5].divisions == (5, 5)
-
-    assert eq(d.loc[5], full.loc[5:5])
-    assert eq(d.loc[3:8], full.loc[3:8])
-    assert eq(d.loc[:8], full.loc[:8])
-    assert eq(d.loc[3:], full.loc[3:])
-
-    assert eq(d.a.loc[5], full.a.loc[5:5])
-    assert eq(d.a.loc[3:8], full.a.loc[3:8])
-    assert eq(d.a.loc[:8], full.a.loc[:8])
-    assert eq(d.a.loc[3:], full.a.loc[3:])
-
-    assert raises(KeyError, lambda: d.loc[1000])
-    assert eq(d.loc[1000:], full.loc[1000:])
-    assert eq(d.loc[-2000:-1000], full.loc[-2000:-1000])
-
-    assert sorted(d.loc[5].dask) == sorted(d.loc[5].dask)
-    assert sorted(d.loc[5].dask) != sorted(d.loc[6].dask)
-
-
-def test_loc_non_informative_index():
-    df = pd.DataFrame({'x': [1, 2, 3, 4]}, index=[10, 20, 30, 40])
-    ddf = dd.from_pandas(df, npartitions=2, sort=True)
-    ddf.divisions = (None,) * 3
-    assert not ddf.known_divisions
-
-    ddf.loc[20:30].compute(get=dask.get)
-
-    assert eq(ddf.loc[20:30], df.loc[20:30])
-
-    df = pd.DataFrame({'x': [1, 2, 3, 4]}, index=[10, 20, 20, 40])
-    ddf = dd.from_pandas(df, npartitions=2, sort=True)
-    assert eq(ddf.loc[20], df.loc[20:20])
-
-
-def test_loc_with_text_dates():
-    A = tm.makeTimeSeries(10).iloc[:5]
-    B = tm.makeTimeSeries(10).iloc[5:]
-    s = dd.Series({('df', 0): A, ('df', 1): B}, 'df', None,
-                  [A.index.min(), B.index.min(), B.index.max()])
-
-    assert s.loc['2000': '2010'].divisions == s.divisions
-    assert eq(s.loc['2000': '2010'], s)
-    assert len(s.loc['2000-01-03': '2000-01-05'].compute()) == 3
-
-
-def test_loc_with_series():
-    assert eq(d.loc[d.a % 2 == 0], full.loc[full.a % 2 == 0])
-
-    assert sorted(d.loc[d.a % 2].dask) == sorted(d.loc[d.a % 2].dask)
-    assert sorted(d.loc[d.a % 2].dask) != sorted(d.loc[d.a % 3].dask)
-
-
-def test_getitem():
-    df = pd.DataFrame({'A': [1, 2, 3, 4, 5, 6, 7, 8, 9],
-                       'B': [9, 8, 7, 6, 5, 4, 3, 2, 1],
-                       'C': [True, False, True] * 3},
-                      columns=list('ABC'))
-    ddf = dd.from_pandas(df, 2)
-    assert eq(ddf['A'], df['A'])
-    tm.assert_series_equal(ddf['A']._pd, ddf._pd['A'])# check cache consistency
-
-    assert eq(ddf[['A', 'B']], df[['A', 'B']])
-    tm.assert_frame_equal(ddf[['A', 'B']]._pd, ddf._pd[['A', 'B']])
-
-    assert eq(ddf[ddf.C], df[df.C])
-    tm.assert_series_equal(ddf.C._pd, ddf._pd.C)
-
-    assert eq(ddf[ddf.C.repartition([0, 2, 5, 8])], df[df.C])
-
-    assert raises(KeyError, lambda: df['X'])
-    assert raises(KeyError, lambda: df[['A', 'X']])
-    assert raises(AttributeError, lambda: df.X)
-
-    # not str/unicode
-    df = pd.DataFrame(np.random.randn(10, 5))
-    ddf = dd.from_pandas(df, 2)
-    assert eq(ddf[0], df[0])
-    assert eq(ddf[[1, 2]], df[[1, 2]])
-
-    assert raises(KeyError, lambda: df[8])
-    assert raises(KeyError, lambda: df[[1, 8]])
-
-
 def test_assign():
     assert eq(d.assign(c=d.a + 1, e=d.a + d.b),
               full.assign(c=full.a + 1, e=full.a + full.b))
@@ -1083,6 +1035,7 @@ def test_set_partition_2():
     assert list(result.compute(get=get_sync).index[-2:]) == ['d', 'd']
 
 
+@pytest.mark.slow
 def test_repartition():
     def _check_split_data(orig, d):
         """Check data is split properly"""
@@ -1306,29 +1259,6 @@ def test_empty_max():
     assert eq(a.x.max(), 1)
 
 
-def test_loc_on_numpy_datetimes():
-    df = pd.DataFrame({'x': [1, 2, 3]},
-                      index=list(map(np.datetime64, ['2014', '2015', '2016'])))
-    a = dd.from_pandas(df, 2)
-    a.divisions = list(map(np.datetime64, a.divisions))
-
-    assert eq(a.loc['2014': '2015'], a.loc['2014': '2015'])
-
-
-def test_loc_on_pandas_datetimes():
-    df = pd.DataFrame({'x': [1, 2, 3]},
-                      index=list(map(pd.Timestamp, ['2014', '2015', '2016'])))
-    a = dd.from_pandas(df, 2)
-    a.divisions = list(map(pd.Timestamp, a.divisions))
-
-    assert eq(a.loc['2014': '2015'], a.loc['2014': '2015'])
-
-
-def test_coerce_loc_index():
-    for t in [pd.Timestamp, np.datetime64]:
-        assert isinstance(_coerce_loc_index([t('2014')], '2014'), t)
-
-
 def test_nlargest_series():
     s = pd.Series([1, 3, 5, 2, 4, 6])
     ss = dd.from_pandas(s, npartitions=2)
@@ -1344,8 +1274,6 @@ def test_query():
         assert eq(q, df.query('x**2 > y'))
 
 
-@pytest.mark.skipif(LooseVersion(pd.__version__) <= '0.18.0',
-                    reason="eval inplace not supported")
 def test_eval():
     p = pd.DataFrame({'x': [1, 2, 3, 4], 'y': [5, 6, 7, 8]})
     d = dd.from_pandas(p, npartitions=2)
@@ -1534,6 +1462,7 @@ def test_corr():
     pytest.raises(TypeError, lambda: da.corr(ddf))
 
 
+@pytest.mark.slow
 def test_cov_corr_stable():
     df = pd.DataFrame(np.random.random((20000000, 2)) * 2 - 1, columns=['a', 'b'])
     ddf = dd.from_pandas(df, npartitions=50)
@@ -1585,8 +1514,6 @@ def test_index_time_properties():
     assert (i.index.month == a.index.month.compute()).all()
 
 
-@pytest.mark.skipif(LooseVersion(pd.__version__) <= '0.16.2',
-                    reason="nlargest not in pandas pre 0.16.2")
 def test_nlargest():
     from string import ascii_lowercase
     df = pd.DataFrame({'a': np.random.permutation(10),
@@ -1598,8 +1525,6 @@ def test_nlargest():
     eq(res, exp)
 
 
-@pytest.mark.skipif(LooseVersion(pd.__version__) <= '0.16.2',
-                    reason="nlargest not in pandas pre 0.16.2")
 def test_nlargest_multiple_columns():
     from string import ascii_lowercase
     df = pd.DataFrame({'a': np.random.permutation(10),
@@ -1759,3 +1684,51 @@ def test_info(capsys):
                   "Data columns (total 2 columns):\n"
                   "long_column_name    int64\n"
                   "short_name          int64\n")
+
+
+def test_gh_1301():
+    df = pd.DataFrame([['1', '2'], ['3', '4']])
+    ddf = dd.from_pandas(df, npartitions=2)
+    ddf2 = ddf.assign(y=ddf[1].astype(int))
+    eq(ddf2,
+       df.assign(y=df[1].astype(int)))
+
+    assert ddf2.dtypes['y'] == np.dtype(int)
+
+
+def test_timeseries_sorted():
+    df = tm.makeTimeDataFrame()
+    ddf = dd.from_pandas(df.reset_index(), npartitions=2)
+    df.index.name = 'index'
+    eq(ddf.set_index('index', sorted=True, drop=True),
+       df)
+
+
+def test_build_pd():
+    s, known_dtype = dd.Series._build_pd(pd.NaT)
+    assert isinstance(s, pd.Series)
+    assert np.issubdtype(s.dtype, np.datetime64)
+
+
+def test_column_assignment():
+    df = pd.DataFrame({'x': [1, 2, 3, 4], 'y': [1, 0, 1, 0]})
+    ddf = dd.from_pandas(df, npartitions=2)
+    from copy import copy
+    orig = copy(ddf)
+    ddf['z'] = ddf.x + ddf.y
+    df['z'] = df.x + df.y
+
+    eq(df, ddf)
+
+
+def test_columns_assignment():
+    df = pd.DataFrame({'x': [1, 2, 3, 4]})
+    ddf = dd.from_pandas(df, npartitions=2)
+
+    df2 = df.assign(y=df.x + 1, z=df.x - 1)
+    df[['a', 'b']] = df2[['y', 'z']]
+
+    ddf2 = ddf.assign(y=ddf.x + 1, z=ddf.x - 1)
+    ddf[['a', 'b']] = ddf2[['y', 'z']]
+
+    eq(df, ddf)
