@@ -1,20 +1,15 @@
 from itertools import product
-from distutils.version import LooseVersion
 
 import pandas as pd
-import pandas.util.testing as tm
 import pytest
 
 from dask.utils import raises
+from dask.dataframe.utils import eq
 import dask.dataframe as dd
 
 
-if LooseVersion(pd.__version__) >= '0.18.0':
-    def resample(df, freq, how='mean', **kwargs):
-        return getattr(df.resample(freq, **kwargs), how)()
-else:
-    def resample(df, freq, how='mean', **kwargs):
-        return df.resample(freq, how=how, **kwargs)
+def resample(df, freq, how='mean', **kwargs):
+    return getattr(df.resample(freq, **kwargs), how)()
 
 
 @pytest.mark.parametrize(['method', 'npartitions', 'freq', 'closed', 'label'],
@@ -26,17 +21,16 @@ else:
 def test_series_resample(method, npartitions, freq, closed, label):
     index = pd.date_range('1-1-2000', '2-15-2000', freq='h')
     index = index.union(pd.date_range('4-15-2000', '5-15-2000', freq='h'))
-    df = pd.Series(range(len(index)), index=index)
-    ds = dd.from_pandas(df, npartitions=npartitions)
+    ps = pd.Series(range(len(index)), index=index)
+    ds = dd.from_pandas(ps, npartitions=npartitions)
     # Series output
+
     result = resample(ds, freq, how=method, closed=closed, label=label)
+    expected = resample(ps, freq, how=method, closed=closed, label=label)
+    eq(result, expected, check_dtype=False)
+
     divisions = result.divisions
-    result = result.compute()
-    expected = resample(df, freq, how=method, closed=closed, label=label)
-    if method != 'ohlc':
-        tm.assert_series_equal(result, expected, check_dtype=False)
-    else:
-        tm.assert_frame_equal(result, expected, check_dtype=False)
+
     assert expected.index[0] == divisions[0]
     assert expected.index[-1] == divisions[-1]
 
