@@ -1,6 +1,7 @@
 from __future__ import absolute_import, division, print_function
 
 import re
+import os
 from functools import partial
 
 from graphviz import Digraph
@@ -52,6 +53,7 @@ def name(x):
 
 
 _HASHPAT = re.compile('([0-9a-z]{32})')
+_UUIDPAT = re.compile('([0-9a-z]{8}-[0-9a-z]{4}-[0-9a-z]{4}-[0-9a-z]{4}-[0-9a-z]{12})')
 
 
 def label(x, cache=None):
@@ -68,21 +70,30 @@ def label(x, cache=None):
     >>> x
     'x-81dc9bdb52d04dc20036dbd8313ed055-hello'
 
+    >>> label(x)
+    'x-#-hello'
+
+    >>> from uuid import uuid1
+    >>> x = 'x-%s-hello' % uuid1()
+    >>> x  # doctest: +SKIP
+    'x-4c1a3d7e-0b45-11e6-8334-54ee75105593-hello'
+
     >>> label(x)
     'x-#-hello'
     """
     s = str(x)
-    m = re.search(_HASHPAT, s)
-    if m is not None:
-        for h in m.groups():
-            if cache is not None:
-                n = cache.get(h, len(cache))
-                label = '#{0}'.format(n)
-                # cache will be overwritten destructively
-                cache[h] = n
-            else:
-                label = '#'
-            s = s.replace(h, label)
+    for pattern in (_HASHPAT, _UUIDPAT):
+        m = re.search(pattern, s)
+        if m is not None:
+            for h in m.groups():
+                if cache is not None:
+                    n = cache.get(h, len(cache))
+                    label = '#{0}'.format(n)
+                    # cache will be overwritten destructively
+                    cache[h] = n
+                else:
+                    label = '#'
+                s = s.replace(h, label)
     return s
 
 
@@ -201,8 +212,8 @@ def dot_graph(dsk, filename='mydask', format=None, **kwargs):
 
     fmts = ['.png', '.pdf', '.dot', '.svg', '.jpeg', '.jpg']
     if format is None and any(filename.lower().endswith(fmt) for fmt in fmts):
-        format = filename.lower().split('.')[-1]
-        filename = filename.rsplit('.')[0]
+        filename, format = os.path.splitext(filename)
+        format = format[1:].lower()
 
     if format is None:
         format = 'png'
