@@ -21,7 +21,7 @@ def test_to_task_dasks():
     a = delayed(1, name='a')
     b = delayed(2, name='b')
     task, dasks = to_task_dasks([a, b, 3])
-    assert task == ['a', 'b', 3]
+    assert task == (list, ['a', 'b', 3])
     assert len(dasks) == 2
     assert a.dask in dasks
     assert b.dask in dasks
@@ -58,6 +58,8 @@ def test_methods():
     assert a.split(' ').compute() == ['a', 'b', 'c', 'd', 'e']
     assert a.upper().replace('B', 'A').split().count('A').compute() == 2
     assert a.split(' ', pure=True).key == a.split(' ', pure=True).key
+    o = a.split(' ', dask_key_name='test')
+    assert o.key == 'test'
 
 
 def test_attributes():
@@ -132,6 +134,8 @@ def test_literates():
     b = a + 1
     lit = (a, b, 3)
     assert delayed(lit).compute() == (1, 2, 3)
+    lit = [a, b, 3]
+    assert delayed(lit).compute() == [1, 2, 3]
     lit = set((a, b, 3))
     assert delayed(lit).compute() == set((1, 2, 3))
     lit = {a: 'a', b: 'b', 3: 'c'}
@@ -142,6 +146,14 @@ def test_literates():
     assert delayed(lit)['a'].compute() == 1
 
 
+def test_literates_keys():
+    a = delayed(1)
+    b = a + 1
+    lit = (a, b, 3)
+    assert delayed(lit).key != delayed(lit).key
+    assert delayed(lit, pure=True).key == delayed(lit, pure=True).key
+
+
 def test_lists_are_concrete():
     a = delayed(1)
     b = delayed(2)
@@ -182,8 +194,13 @@ def test_kwargs():
     ten = dmysum(1, 2, c=[delayed(3), 0], four=dmysum(2, 2))
     assert ten.compute() == 10
     dmysum = delayed(mysum, pure=True)
-    ten = dmysum(1, 2, c=[delayed(3), 0], four=dmysum(2, 2))
+    c = [delayed(3), 0]
+    ten = dmysum(1, 2, c=c, four=dmysum(2, 2))
     assert ten.compute() == 10
+    assert dmysum(1, 2, c=c, four=dmysum(2, 2)).key == ten.key
+    assert dmysum(1, 2, c=c, four=dmysum(2, 3)).key != ten.key
+    assert dmysum(1, 2, c=c, four=4).key != ten.key
+    assert dmysum(1, 2, c=c, four=4).key != dmysum(2, 2, c=c, four=4).key
 
 
 def test_array_delayed():
@@ -298,3 +315,8 @@ def test_name_consitent_across_instances():
 def test_sensitive_to_partials():
     assert (delayed(partial(add, 10), pure=True)(2)._key !=
             delayed(partial(add, 20), pure=True)(2)._key)
+
+
+def test_value_name():
+    assert delayed(1)._key.startswith('int-')
+    assert delayed(1, pure=True)._key.startswith('int-')
