@@ -1,18 +1,20 @@
+import io
 import os
 
 import numpy as np
 import pytest
 
-from dask.compatibility import BZ2File, GzipFile, LZMAFile
+from dask.compatibility import BZ2File, GzipFile, LZMAFile, LZMA_AVAILABLE
 from dask.utils import (textblock, filetext, takes_multiple_arguments,
-                        Dispatch, tmpfile, different_seeds)
+                        Dispatch, tmpfile, different_seeds, file_size,
+                        infer_storage_options)
 
 
+SKIP_XZ = pytest.mark.skipif(not LZMA_AVAILABLE, reason="no lzma library")
 @pytest.mark.parametrize('myopen,compression',
-                         [(open, None), (GzipFile, 'gzip'), (BZ2File, 'bz2'), (LZMAFile, 'xz')])
+                         [(open, None), (GzipFile, 'gzip'), (BZ2File, 'bz2'),
+                          SKIP_XZ((LZMAFile, 'xz'))])
 def test_textblock(myopen, compression):
-    if compression == 'xz':
-        pytest.importorskip('lzma')
     text = b'123 456 789 abc def ghi'.replace(b' ', os.linesep.encode())
     with filetext(text, open=myopen, mode='wb') as fn:
         text = ''.join(textblock(fn, 1, 11, compression)).encode()
@@ -24,6 +26,15 @@ def test_textblock(myopen, compression):
         assert ''.join(textblock(fn, k, k, compression)).encode() == b''
 
 
+@pytest.mark.parametrize('myopen,compression',
+                         [(open, None), (GzipFile, 'gzip'), (BZ2File, 'bz2'),
+                          SKIP_XZ((LZMAFile, 'xz'))])
+def test_filesize(myopen, compression):
+    text = b'123 456 789 abc def ghi'.replace(b' ', os.linesep.encode())
+    with filetext(text, open=myopen, mode='wb') as fn:
+        assert file_size(fn, compression) == len(text)
+
+
 def test_textblock_multibyte_linesep():
     text = b'12 34 56 78'.replace(b' ', b'\r\n')
     with filetext(text, mode='wb') as fn:
@@ -110,3 +121,37 @@ def test_different_seeds():
     # Should be sorted
     smallseeds = different_seeds(10, 1234)
     assert smallseeds == sorted(smallseeds)
+
+
+def test_infer_storage_options():
+    so = infer_storage_options('/mnt/datasets/test.csv')
+    assert so.pop('protocol') == 'file'
+    assert so.pop('path') == '/mnt/datasets/test.csv'
+    assert not so
+
+    assert infer_storage_options('./test.csv')['path'] == './test.csv'
+    assert infer_storage_options('../test.csv')['path'] == '../test.csv'
+
+    so = infer_storage_options(
+              'hdfs://username:pwd@node:123/mnt/datasets/test.csv?q=1#fragm',
+              inherit_storage_options={'extra': 'value'})
+    assert so.pop('protocol') == 'hdfs'
+    assert so.pop('username') == 'username'
+    assert so.pop('password') == 'pwd'
+    assert so.pop('host') == 'node'
+    assert so.pop('port') == 123
+    assert so.pop('path') == '/mnt/datasets/test.csv'
+    assert so.pop('url_query') == 'q=1'
+    assert so.pop('url_fragment') == 'fragm'
+    assert so.pop('extra') == 'value'
+    assert not so
+
+    with pytest.raises(KeyError):
+        infer_storage_options('file:///bucket/file.csv', {'path': 'collide'})
+    with pytest.raises(KeyError):
+        infer_storage_options('hdfs:///bucket/file.csv', {'protocol': 'collide'})
+
+
+def test_infer_storage_options_c():
+    so = infer_storage_options(r'c:\foo\bar')
+    assert so['protocol'] == 'file'
